from rest_framework import viewsets, status, response, generics, permissions
from django.db.models import Q
# from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import filters
from django.shortcuts import get_object_or_404
from blog.serializers import *
from blog.utils import ActionBasedPermission


class ArticleCategoryViewSet(viewsets.ModelViewSet):
    '''This ViewSet class is for managing the CRUD operations on ArticleCategory model'''
    serializer_class = ArticleCategorySerializer
    queryset = ArticleCategory.objects.all()
    lookup_field = 'slug'

    def list(self, request, *args, **kwargs):
        '''
        if it found 'status' in 'query_params' then it will return categories list of the current status
        otherwise it will return only active categories list
        '''
        queryset = self.get_queryset().filter(status__exact=request.GET.get('status', None) or 'active')
        return response.Response(self.serializer_class(queryset, many=True).data, status=status.HTTP_200_OK)


class ArticleSubcategoryViewSet(viewsets.ModelViewSet):
    '''This ViewSet class is for managing the CRUD operations on ArticleSubcategory model'''
    serializer_classes = {
        'create': CreateArticleSubcategorySerializer,
        'list': ListArticleSubcategorySerializer,
        'retrieve': ListArticleSubcategorySerializer,
        'update': CreateArticleSubcategorySerializer
    }
    default_serializer_class = ListArticleSubcategorySerializer
    queryset = ArticleSubcategory.objects.all()
    lookup_field = 'slug'

    def get_serializer_class(self):
        return self.serializer_classes.get(self.action, self.default_serializer_class)

    def list(self, request, *args, **kwargs):
        '''
        if it found 'category_slug' and 'status' in 'query_params' then it will return the list of subcategories of this
        given category and the given status otherwise it will return the list of active subcategories
        '''
        queryset = self.get_queryset().filter(Q(category__slug__exact=request.GET.get('category_slug')) &
                                              Q(status__exact=request.GET.get('status', None) or 'active')) \
            if request.GET.get('category_slug', None) else self.get_queryset().filter(status__exact=request.GET.
                                                                                      get('status', None) or 'active')
        return response.Response(self.get_serializer_class()(queryset, many=True).data, status=status.HTTP_200_OK)

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer_class()(data=request.data)
        serializer.is_valid(raise_exception=True)
        if serializer.validated_data['category'].status != 'active': return response.Response(
            {'detail': 'Category is inactive.'}, status=status.HTTP_400_BAD_REQUEST)
        serializer.save()
        return response.Response(serializer.data, status=status.HTTP_201_CREATED)

    def retrieve(self, request, *args, **kwargs):
        return response.Response(self.get_serializer_class()(get_object_or_404(ArticleSubcategory, slug=kwargs['slug']),
                                                             many=False).data, status=status.HTTP_200_OK)

    def update(self, request, *args, **kwargs):
        query = get_object_or_404(ArticleSubcategory, slug=kwargs['slug'])
        serializer = self.get_serializer_class()(query, data=request.data, partial=True)
        serializer.is_valid()
        if 'category' in serializer.validated_data and serializer.validated_data['category'].status != 'active':
            return response.Response({'detail': 'Category is inactive'}, status=status.HTTP_400_BAD_REQUEST)
        serializer.save()
        return response.Response(serializer.data, status=status.HTTP_200_OK)


class ArticleViewSet(viewsets.ModelViewSet):
    '''This ViewSet class is for managing the CRUD operations on Article model'''
    serializer_classes = {
        'create': CreateArticleSerializer,
        'list': ListArticleSerializer,
        'retrieve': ListArticleSerializer,
        'update': CreateArticleSerializer
    }
    default_serializer_class = ListArticleSerializer
    permission_classes = [permissions.AllowAny, ]
    queryset = Article.objects.all()
    lookup_field = 'slug'

    def get_serializer_class(self, *args, **kwargs):
        return self.serializer_classes.get(self.action, self.default_serializer_class)

    def list(self, request, *args, **kwargs):
        '''
        1. if it found 'subcategory_slug' and 'status' then it will return all the articles of this given category and given status
        2. if it found only 'subcategory_slug' then it will return all the active articles of this given category
        3. if it found only 'status' then it will return all the articles of the given status
        4. if it does not found 'status' and 'subcategory_slug' then it will return all the active articles
        '''
        queryset = self.get_queryset().filter(
            Q(subcategory__slug=request.GET.get('subcategory_slug') &
                                Q(status__exact=request.GET.get('status', None) or 'active'))) \
            if request.GET.get('subcategory_slug', None) \
            else self.get_queryset().filter(status__exact=request.GET.get('status', None) or 'active')
        return response.Response(self.get_serializer_class()(queryset, many=True).data,
                                 status=status.HTTP_200_OK)

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer_class()(data=request.data)
        serializer.is_valid(raise_exception=True)
        if serializer.validated_data['subcategory'].status != 'active':
            return response.Response({'detail': 'Subcategory is inactive'}, status=status.HTTP_400_BAD_REQUEST)
        serializer.save()
        return response.Response(serializer.data, status=status.HTTP_200_OK)

    def retrieve(self, request, *args, **kwargs):
        return response.Response(self.get_serializer_class()(get_object_or_404(Article, slug=kwargs['slug']),
                                                             many=False).data, status=status.HTTP_200_OK)

    def update(self, request, *args, **kwargs):
        query = get_object_or_404(Article, slug=kwargs['slug'])
        serializer = self.get_serializer_class()(query, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        if 'subcategory' in serializer.validated_data and serializer.validated_data['subcategory'].status != 'active':
            return response.Response({'detail': 'Subcategory is inactive'}, status=status.HTTP_400_BAD_REQUEST)
        serializer.save()
        return response.Response(serializer.data, status=status.HTTP_200_OK)


class RelatedArticlesListAPI(generics.GenericAPIView):
    '''This API will return related_articles, it accepts article slug in query params'''
    serializer_class = ListArticleSerializer
    queryset = Article.objects.all()

    def get(self, request, *args, **kwargs):
        if not request.GET.get('slug', None): return response.Response({'detail': 'Article slug is required in params'},
                                                                       status=status.HTTP_400_BAD_REQUEST)
        query = get_object_or_404(Article, slug=request.GET.get('slug'))
        queryset = query.tags.similar_objects()
        [queryset.remove(article) for article in queryset if article.status != 'active']
        serializer = self.serializer_class(queryset, many=True)
        return response.Response(serializer.data, status=status.HTTP_200_OK)


class ArticleLikeAndDislikeAPI(generics.GenericAPIView):
    '''
    This API is used to like and dislike on article
    '''
    serializer_class = LikeAndDislikeSerializer
    permission_classes = [permissions.IsAuthenticated, ]

    def post(self, request, *args, **kwargs):
        payload = request.data.copy()
        payload['user'] = request.user.id
        serializer = self.serializer_class(data=payload)
        serializer.is_valid(raise_exception=True)
        return response.Response(serializer.validated_data, status=status.HTTP_200_OK)


class ArticleCommentViewSet(viewsets.ModelViewSet):
    '''
    This API is for managing the CRUD operations on Comment model
    '''
    serializer_classes = {
        'list': ListCommentSerializer,
        'create': CreateCommentSerializer
    }
    default_serializer_class = ListCommentSerializer
    queryset = Comment.objects.all()
    permission_classes = (ActionBasedPermission,)
    action_permissions = {
        permissions.IsAuthenticated: ['update', 'partial_update', 'destroy', 'create'],
        permissions.AllowAny: ['retrieve', 'list']
    }
    lookup_field = 'pk'

    def get_serializer_class(self):
        return self.serializer_classes.get(self.action, self.default_serializer_class)

    def list(self, request, *args, **kwargs):
        '''
        It accepts 'article_id' in params and return all the active comments of the given article
        '''
        blog_id = request.GET.get('article_id', None)
        if not blog_id: return response.Response({'detail': 'article_id required in params.'},
                                                 status=status.HTTP_400_BAD_REQUEST)
        return response.Response(self.get_serializer_class()(get_object_or_404(Article, pk=int(blog_id)).comments.
                                                             filter(status__exact='active'), many=True).data,
                                 status=status.HTTP_200_OK)

    def create(self, request, *args, **kwargs):
        payload = request.data.copy()
        payload['created_by'] = request.user.id
        serializer = self.get_serializer_class()(data=payload)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return response.Response(serializer.data, status=status.HTTP_201_CREATED)

    def update(self, request, *args, **kwargs):
        query = get_object_or_404(Comment, pk=kwargs['pk'])
        if query.user != request.user:
            return response.Response({'detail': 'Permission denied.'}, status=status.HTTP_403_FORBIDDEN)
        serializer = self.get_serializer_class()(query, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return response.Response(serializer.data, status=status.HTTP_200_OK)

    def destroy(self, request, *args, **kwargs):
        query = get_object_or_404(Comment, pk=kwargs['pk'])
        if query.user != request.user:
            return response.Response({'detail': 'Permission denied.'}, status=status.HTTP_403_FORBIDDEN)
        query.delete()
        return response.Response({'detail': 'Comment deleted successfully'}, status=status.HTTP_204_NO_CONTENT)


class ArticleSearchAPI(generics.ListAPIView):
    '''This API is to search the articles'''
    serializer_class = ListArticleSerializer
    queryset = Article.objects.filter(status__exact='active')
    filter_backends = [filters.SearchFilter]
    search_fields = ['subcategory__title', 'subcategory__slug', 'title', 'author_name', 'author__first_name',
                     'author__last_name', 'author__username', 'author__email', 'keywords', 'seo_title', 'tags__name']





from rest_framework import serializers, exceptions
from taggit.models import Tag
from django.shortcuts import get_object_or_404
from blog.models import ArticleCategory, ArticleSubcategory, Article, User, Comment


class ArticleCategorySerializer(serializers.ModelSerializer):
    class Meta:
        model = ArticleCategory
        fields = '__all__'


class CreateArticleSubcategorySerializer(serializers.ModelSerializer):
    class Meta:
        model = ArticleSubcategory
        fields = '__all__'


class ListArticleSubcategorySerializer(serializers.ModelSerializer):
    '''This serializer is to return the proper data of list and retrieve apis'''
    category = ArticleCategorySerializer(many=False)

    class Meta:
        model = ArticleSubcategory
        fields = '__all__'


class TagSerializer(serializers.ModelSerializer):
    class Meta:
        model = Tag
        fields = '__all__'


class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = ['id', 'first_name', 'last_name', 'username', 'email']


class CreateArticleSerializer(serializers.ModelSerializer):
    tags = TagSerializer(many=True)

    class Meta:
        model = Article
        fields = '__all__'


class ListArticleSerializer(serializers.ModelSerializer):
    subcategory = ListArticleSubcategorySerializer(many=False)
    author = serializers.SerializerMethodField('get_author_full_name')
    tags = TagSerializer(many=True)
    like = serializers.SerializerMethodField('get_total_likes')
    comments = serializers.SerializerMethodField('get_total_comments')

    def get_total_likes(self, instance):
        return instance.like.all().count()

    def get_total_comments(self, instance):
        return instance.comments.all().count()

    def get_author_full_name(self, instance):
        try:
            return f'{self.author.first_name} {self.author.last_name}'
        except:
            return None

    class Meta:
        model = Article
        fields = '__all__'


class LikeAndDislikeSerializer(serializers.Serializer):
    '''This Serializer Class is for like and dislike on articles'''
    user = serializers.IntegerField()
    article = serializers.IntegerField()

    def validate(self, attrs):
        article = get_object_or_404(Article, id=attrs['article'])
        article.like.remove(attrs['user']) if article.like.filter(pk=attrs['user']).last() else article.like.add(
            attrs['user'])
        return {'likes': article.total_likes()}


class CreateCommentSerializer(serializers.Serializer):
    article = serializers.IntegerField()
    user = serializers.IntegerField()
    body = serializers.CharField()

    def validate(self, attrs):
        article = get_object_or_404(Article, id=attrs['article'])
        if article.allow_comments is False:
            raise exceptions.PermissionDenied('Comments are not allowed for this article')
        response_data = {
            'article': article,
            'user': User.objects.get(pk=attrs['user']),
            'body': attrs['body']
        }
        return response_data

    def create(self, validated_data):
        new_comment = Comment.objects.create(user=validated_data['user'], body=validated_data['body'])
        validated_data['article'].comments.add(new_comment)
        res = {
            'article': validated_data['article'].id,
            'user': validated_data['user'].id,
            'body': validated_data['body']
        }
        return res


class ListCommentSerializer(serializers.ModelSerializer):
    user = UserSerializer(many=False)

    class Meta:
        model = Comment
        fields = '__all__'




from django.urls import path, include
from rest_framework.routers import DefaultRouter
from blog.views import *

router = DefaultRouter()
router.register('categories', ArticleCategoryViewSet, basename='article_category')
router.register('subcategories', ArticleSubcategoryViewSet, basename='article_subcategory')
router.register('article', ArticleViewSet, basename='article')
router.register('comments', ArticleCommentViewSet, basename='comments')

urlpatterns = [
    path('', include(router.urls)),
    path('related-articles', RelatedArticlesListAPI.as_view(), name='related_articles'),
    path('like-and-dislike/', ArticleLikeAndDislikeAPI.as_view(), name='like_and_dislike'),
    path('search', ArticleSearchAPI.as_view(), name='search'),
]




# Generated by Django 4.0.2 on 2022-02-11 05:58

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('blog', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='article',
            name='allow_comments',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='article',
            name='author_name',
            field=models.CharField(blank=True, help_text='Ex. John Doe', max_length=150, null=True, verbose_name='Custom author name'),
        ),
        migrations.AddField(
            model_name='article',
            name='like',
            field=models.ManyToManyField(blank=True, null=True, related_name='like', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='article',
            name='view',
            field=models.ManyToManyField(blank=True, null=True, related_name='view', to=settings.AUTH_USER_MODEL),
        ),
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('abstractcommonfields_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='blog.abstractcommonfields')),
                ('body', models.TextField()),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Comment',
                'verbose_name_plural': 'Comments',
                'ordering': ['-id'],
            },
            bases=('blog.abstractcommonfields',),
        ),
        migrations.AddField(
            model_name='article',
            name='comments',
            field=models.ManyToManyField(blank=True, null=True, related_name='comment', to='blog.Comment'),
        ),
    ]



from django.db import models
from ckeditor_uploader.fields import RichTextUploadingField
from taggit.managers import TaggableManager
from django.contrib.auth import get_user_model
from django.dispatch import receiver
from django.db.models.signals import pre_save
from blog.helpers import slug_generator


User = get_user_model()


class AbstractCommonFields(models.Model):
    STATUS_CHOICES = (
        ('active', 'Active'),
        ('inactive', 'Inactive')
    )
    status = models.CharField(max_length=10, choices=STATUS_CHOICES, default='active')
    created_on = models.DateTimeField(auto_now_add=True)
    updated_on = models.DateTimeField(auto_now=True)

    abstract = True


class ArticleCategory(AbstractCommonFields):
    title = models.CharField(max_length=100, unique=True, primary_key=False, verbose_name='category name',
                             help_text='Category name length should be in 100 characters')
    slug = models.SlugField(max_length=255, unique=True, primary_key=False, blank=True, null=True)

    def __str__(self):
        return self.title

    class Meta:
        verbose_name = 'Category'
        verbose_name_plural = 'Categories'
        ordering = ['-id', ]


@receiver(pre_save, sender=ArticleCategory)
def _pre_save_receiver(sender, instance, *args, **kwargs):
    try:
        old_value = sender.objects.get(id=instance.id).title
        if old_value != instance.title:instance.slug = slug_generator(instance)
    except:
        instance.slug = slug_generator(instance)


class ArticleSubcategory(AbstractCommonFields):
    category = models.ForeignKey(ArticleCategory, on_delete=models.RESTRICT, help_text='Only active categories will be '
                                                                                       'displayed here')
    title = models.CharField(max_length=100, unique=True, primary_key=False, verbose_name='subcategory name',
                             help_text='Subcategory name length should be in 100 characters')
    slug = models.SlugField(max_length=255, blank=True, null=True, unique=True, primary_key=False)

    def __str__(self):
        return self.title

    class Meta:
        verbose_name = 'Subcategory'
        verbose_name_plural = 'Subcategories'
        ordering = ['-id', ]


@receiver(pre_save, sender=ArticleSubcategory)
def _pre_save_receiver(sender, instance, *args, **kwargs):
    try:
        old_value = sender.objects.get(id=instance.id).title
        if old_value != instance.title: instance.slug = slug_generator(instance)
    except:
        instance.slug = slug_generator(instance)


class Article(AbstractCommonFields):
    subcategory = models.ForeignKey(ArticleSubcategory, on_delete=models.RESTRICT, help_text='Only active subcategories'
                                                                                             ' will displayed here')
    title = models.CharField(max_length=200, help_text='Blog title length should be within 200')
    slug = models.SlugField(max_length=255, blank=True, null=True, unique=True, primary_key=False)
    author = models.ForeignKey(User, on_delete=models.CASCADE, related_name='blog_posts', blank=True,
                               null=True)
    author_name = models.CharField(max_length=150, blank=True, null=True, verbose_name='Custom author name',
                                   help_text='Ex. John Doe')
    introduction = models.TextField(blank=True, null=True)
    description = models.TextField(blank=True, null=True)
    body = RichTextUploadingField()
    thumbnail = models.ImageField(upload_to='articles/thumbnails/', blank=True, null=True)
    cover_image = models.ImageField(upload_to='articles/cover-images/', blank=True, null=True)
    image_alt_tag = models.CharField(max_length=50, blank=True, null=True)
    image_caption = models.CharField(max_length=50, blank=True, null=True)
    keywords = models.CharField(max_length=255, help_text='Keywords must be separated by comma(,)')
    tags = TaggableManager(blank=True)
    seo_title = models.CharField(max_length=200, blank=True, null=True)
    seo_introduction = models.TextField(blank=True, null=True)
    seo_description = models.TextField(blank=True, null=True)
    seo_keywords = models.CharField(max_length=255, blank=True, null=True,
                                    help_text='SEO-Keywords must be separated by comma(,)')
    display_date = models.CharField(max_length=50, blank=True, null=True, verbose_name='Custom publish date',
                                    help_text='Ex. 12th March, 2022')
    like = models.ManyToManyField(User, blank=True, null=True, related_name='like')
    comments = models.ManyToManyField('Comment', blank=True, null=True, related_name='comment')
    allow_comments = models.BooleanField(default=False)

    def total_likes(self):
        return self.like.all().count()

    def total_views(self):
        return self.view.all().count()

    def __str__(self):
        return self.title

    def save(self, *args, **kwargs):
        self.seo_title = self.seo_title or self.title
        self.introduction = self.seo_introduction or self.introduction
        self.seo_description = self.seo_description or self.description
        self.seo_keywords = self.seo_keywords or self.keywords
        super(Article, self).save(*args, **kwargs)

    class Meta:
        verbose_name = 'Article'
        verbose_name_plural = 'Articles'
        ordering = ['-created_on', ]


@receiver(pre_save, sender=Article)
def _pre_save_receiver(sender, instance, *args, **kwargs):
    try:
        old_value = sender.objects.get(id=instance.id).title
        if old_value != instance.title: instance.slug = slug_generator(instance)
    except:
        instance.slug = slug_generator(instance)


class Comment(AbstractCommonFields):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    body = models.TextField()

    def __str__(self):
        return f'Comment by: {self.user.first_name} {self.user.last_name}, body: {self.body}'

    class Meta:
        verbose_name = 'Comment'
        verbose_name_plural = 'Comments'
        ordering = ['-id', ]





from rest_framework import permissions


class ActionBasedPermission(permissions.AllowAny):
    """
    Grant or deny access to a view, based on a mapping in view.action_permissions
    """
    def has_permission(self, request, view):
        for klass, actions in getattr(view, 'action_permissions', {}).items():
            if view.action in actions:
                return klass().has_permission(request, view)
        return False




import random2
import string
from django.utils.text import slugify


def random_string_generator(size=10, chars=string.ascii_lowercase + string.digits):
    return ''.join(random2.choice(chars) for _ in range(size))


def slug_generator(instance, new_slug=None):
    if new_slug is not None:
        slug = new_slug
    else:
        slug = slugify(instance.title)
    klass = instance.__class__

    if klass.objects.filter(slug=slug).exists():
        new_slug = "{slug}-{randstr}".format(slug=slug, randstr=random_string_generator(size=4))
        return slug_generator(instance, new_slug=new_slug)
    return slug








from setuptools import setup


setup(
    name='django-customized-blog-package',
    version='1.0.1',
    description='This package is only used for Blog projects using django',
    long_description_content_type='text/x-rst',
    url='https://github.com/AyanNandaGoswami/BlogModule.git',
    author='Ayan Nanda Goswami',
    author_email='ayan02472@gmail.com',
    maintainer='Ayan Nanda Goswami',
    license='MIT',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords='django customized blog package',
    project_urls={
        'Documentation': 'https://blog-module-community.herokuapp.com/',
        'Source': 'https://github.com/AyanNandaGoswami/BlogModule.git',
        'Tracker': 'https://github.com/AyanNandaGoswami/BlogModule/issues',
    },
    install_requires=['Django', 'django-ckeditor', 'django-taggit', 'djangorestframework', 'Pillow', 'random2'],
    python_requires='>=3',
)



from django.contrib import admin
from blog.models import ArticleCategory, ArticleSubcategory, Article, Comment


@admin.register(ArticleCategory)
class ArticleCategoryAdmin(admin.ModelAdmin):
    list_display = ['title', 'id', 'slug', 'status', 'created_on', 'updated_on']
    readonly_fields = ['id', 'created_on', 'updated_on']
    search_fields = ['title', 'slug', 'id']
    search_help_text = ['Search by category name, slug and ID']
    list_display_links = ['title']
    list_per_page = 20


@admin.register(ArticleSubcategory)
class ArticleSubcategoryAdmin(admin.ModelAdmin):
    list_display = ['title', 'id', 'slug', 'category', 'status', 'created_on', 'updated_on']
    readonly_fields = ['id', 'created_on', 'updated_on']
    search_fields = ['title', 'slug', 'id']
    search_help_text = ['Search by subcategory name, slug and ID']
    list_display_links = ['title']
    list_per_page = 20
    list_filter = ['category']

    def get_form(self, request, obj=None, **kwargs):
        '''Override get_form method to get only active categories'''
        form = super(ArticleSubcategoryAdmin, self).get_form(request, obj, **kwargs)
        form.base_fields['category'].queryset = ArticleCategory.objects.filter(status__exact='active')
        return form


@admin.register(Article)
class ArticleAdmin(admin.ModelAdmin):
    list_display = ['title', 'id', 'status', 'created_on', 'updated_on']
    readonly_fields = ['id', 'created_on', 'updated_on']
    search_fields = ['title', 'slug', 'id', 'seo_title', 'keywords']
    search_help_text = ['Search by category name, slug, ID, seo-title, keywords']
    list_display_links = ['title']
    list_per_page = 50

    def get_form(self, request, obj=None, **kwargs):
        '''Override get_form method to get only active subcategories'''
        form = super(ArticleAdmin, self).get_form(request, obj, **kwargs)
        form.base_fields['subcategory'].queryset = ArticleSubcategory.objects.filter(status__exact='active')
        return form


@admin.register(Comment)
class CommentAdmin(admin.ModelAdmin):
    list_display = ['id', 'body', 'user', 'created_on', 'updated_on']




# Generated by Django 4.0.2 on 2022-02-06 17:59

import ckeditor_uploader.fields
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import taggit.managers


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('taggit', '0004_alter_taggeditem_content_type_alter_taggeditem_tag'),
    ]

    operations = [
        migrations.CreateModel(
            name='AbstractCommonFields',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(choices=[('active', 'Active'), ('inactive', 'Inactive')], default='active', max_length=10)),
                ('created_on', models.DateTimeField(auto_now_add=True)),
                ('updated_on', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='ArticleCategory',
            fields=[
                ('abstractcommonfields_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='blog.abstractcommonfields')),
                ('title', models.CharField(help_text='Category name length should be in 100 characters', max_length=100, unique=True, verbose_name='category name')),
                ('slug', models.SlugField(blank=True, max_length=255, null=True, unique=True)),
            ],
            options={
                'verbose_name': 'Category',
                'verbose_name_plural': 'Categories',
                'ordering': ['-id'],
            },
            bases=('blog.abstractcommonfields',),
        ),
        migrations.CreateModel(
            name='ArticleSubcategory',
            fields=[
                ('abstractcommonfields_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='blog.abstractcommonfields')),
                ('title', models.CharField(help_text='Subcategory name length should be in 100 characters', max_length=100, unique=True, verbose_name='subcategory name')),
                ('slug', models.SlugField(blank=True, max_length=255, null=True, unique=True)),
                ('category', models.ForeignKey(help_text='Only active categories will be displayed here', on_delete=django.db.models.deletion.RESTRICT, to='blog.articlecategory')),
            ],
            options={
                'verbose_name': 'Subcategory',
                'verbose_name_plural': 'Subcategories',
                'ordering': ['-id'],
            },
            bases=('blog.abstractcommonfields',),
        ),
        migrations.CreateModel(
            name='Article',
            fields=[
                ('abstractcommonfields_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='blog.abstractcommonfields')),
                ('title', models.CharField(help_text='Blog title length should be within 200', max_length=200)),
                ('slug', models.SlugField(blank=True, max_length=255, null=True, unique=True)),
                ('introduction', models.TextField(blank=True, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('body', ckeditor_uploader.fields.RichTextUploadingField()),
                ('thumbnail', models.ImageField(blank=True, null=True, upload_to='articles/thumbnails/')),
                ('cover_image', models.ImageField(blank=True, null=True, upload_to='articles/cover-images/')),
                ('image_alt_tag', models.CharField(blank=True, max_length=50, null=True)),
                ('image_caption', models.CharField(blank=True, max_length=50, null=True)),
                ('keywords', models.CharField(help_text='Keywords must be separated by comma(,)', max_length=255)),
                ('seo_title', models.CharField(blank=True, max_length=200, null=True)),
                ('seo_introduction', models.TextField(blank=True, null=True)),
                ('seo_description', models.TextField(blank=True, null=True)),
                ('seo_keywords', models.CharField(blank=True, help_text='SEO-Keywords must be separated by comma(,)', max_length=255, null=True)),
                ('display_date', models.CharField(blank=True, help_text='Ex. 12th March, 2022', max_length=50, null=True, verbose_name='Custom publish date')),
                ('author', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='blog_posts', to=settings.AUTH_USER_MODEL)),
                ('subcategory', models.ForeignKey(help_text='Only active subcategories will displayed here', on_delete=django.db.models.deletion.RESTRICT, to='blog.articlesubcategory')),
                ('tags', taggit.managers.TaggableManager(blank=True, help_text='A comma-separated list of tags.', through='taggit.TaggedItem', to='taggit.Tag', verbose_name='Tags')),
            ],
            options={
                'verbose_name': 'Article',
                'verbose_name_plural': 'Articles',
                'ordering': ['-created_on'],
            },
            bases=('blog.abstractcommonfields',),
        ),
    ]
